#!/usr/bin/env python

# This file is part of 'Cut The Crap (CTC)'
# Copyright 1999 Constantinos A. Kotsokalis <ckotso@cs.ntua.gr>
#
# CTC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# CTC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CTC; see the file COPYING. If not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

import sys
import os
import signal
import network_engine
import error_object
import handler
import read_conf

# Instantiate error reporting object, needed by the network engine
errobj = error_object.error_object()

def process_starts():
	pid = os.getpid()
	try:
		runfile = open("/var/run/ctc.pid", "w")
		runfile.write(`pid`)
		runfile.close()
	except IOError, msg:
		errobj.eprint("Could not open file var/run/ctc.pid for writing: ", msg)


def process_ends():
	if posixpath.exists("/var/run/ctc.pid"):
		try:
			os.unlink("/var/run/ctc.pid")
		except OSError, msg:
			errobj.eprint("/var/run/ctc.pid: ", msg[1])
	sys.exit(0)


def reconfigure():
	global HCh, confobj, neteng
	HCh = handler.handler(confobj.conf, neteng, errobj)
	signal.signal(signal.SIGHUP, reconfigure)


if __name__ == '__main__':

	if len(sys.argv) > 1:
		config_file = sys.argv[1]
	else:
		config_file = None
	# Instantiate configuration object
	confobj = read_conf.read_conf(config_file, errobj)
	# Instantiate the network engine
	neteng = network_engine.network_engine(confobj.conf, errobj)
	# Instantiate handler object
	HCh = handler.handler(confobj.conf, neteng, errobj)

	process_starts()

	signal.signal(signal.SIGTERM, process_ends)
	signal.signal(signal.SIGHUP, reconfigure)

	# Go into main loop. The on* functions are defined in handler.py.
	sys.stdout.write("'Cut The Crap' version " + confobj.conf['VERSION'] + \
	        " (" + confobj.conf['VERSION_NAME'] + ") going active...\n")
	neteng.loop(HCh.onAccept, HCh.onRead, HCh.onClose, HCh.onLoopEnd)

