# This file is part of 'Cut The Crap (CTC)'
# Copyright 1999 Constantinos A. Kotsokalis <ckotso@cs.ntua.gr>
#
# CTC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# CTC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CTC; see the file COPYING. If not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

# Event handlers:
# onAccept: Executed when an inbound connection is established (accepted)
# onRead: Executed when data is read from an active connection
# onClose: Executed after a socket closes connection, or we close on with
#			network_engine.sock_close(socket)
# onLoopEnd: Executed after reading data from all sockets available. This
#			mainly for maintenance reasons.

import zaplet_engine
import sockpair
import re
import string
import urlparse
import time
import sys

class handler:

	def __init__(self, conf, neteng, errobj):
		self.conf = conf
		self.neteng = neteng
		self.errobj = errobj
		self.socklist = []
		self.acl = conf['ACL']
		self.z_engine = zaplet_engine.zaplet_engine(conf['ZAPLET_DIR'])
		self.conntype = re.compile("connection: keep-alive", re.I)
		self.gifloc = re.compile(".*/_CTC-stuff/CTC.gif")
		# For future use
		# self.jscript = re.compile("(<SCRIPT LANGUAGE=\"j.{0,3}script\">.*\
									# </SCRIPT>){1,1}", re.I)
		self.popupblk = re.compile("window\.open\s*(\(.*\n*.*\))+")
		self.HTTP_METHODS = ("GET", "POST", "PUT", "HEAD", "MOVE", \
							 "OPTIONS", "DELETE", "TRACE", "CONNECT")
		self.pairs = {}


	def onAccept(self, sock, where_from):
		if not self.acl.is_allowed(where_from[0]):
			self.neteng.sock_close(sock)
		else:
			if self.conf['NO_LOGGING'] != 1:
				sys.stdout.write(string.join([time.ctime(time.time()), ": ", where_from[0], "\n"], ""))
			self.pairs[sock] = sockpair.SockPair(sockpair.CLIENT)

		return


	def onRead(self, sock, buff):
		net_engine = self.neteng
		z_engine = self.z_engine
		pairs = self.pairs
		conntype = self.conntype
		gifloc = self.gifloc
		# jscript = self.jscript
		popupblk = self.popupblk
		cfg = self.conf

		if pairs[sock].type == sockpair.SERVER:
			other_end = pairs[sock].pair
			if other_end == None:
				del(pairs[sock])
				net_engine.sock_close(sock)
			else:
				if conntype.search(buff) != None:
					buff = conntype.sub("Proxy-Connection: Close", buff)
				# For future use
				# if cfg['BLOCK_POPUPS'] and jscript.search(buff) != None:
					# jlist = jscript.findall(buff)
					# for str in jlist:
						# if popupblk.search(str) != None:
							# wlist = popupblk.findall(str)
							# for wstr in wlist:
								# str = popupblk.sub('', str)
						# buff = jscript.sub(str, buff)
				if cfg['BLOCK_POPUPS'] and popupblk.search(buff) != None:
					buff = popupblk.sub('', buff)
				net_engine.sock_write(other_end, buff)
		else:
			tokens = string.split(buff)
			# Will have to think about this: POST operations might
			# take place in two steps.
			if len(tokens) < 1:
				return

			if tokens[0] in self.HTTP_METHODS:
				try:
		 			if z_engine.urlblocker(tokens[1]):
						# send blank gif
						net_engine.sock_write(sock, cfg['ADV_RET_BUFF'])
						net_engine.sock_write(sock, cfg['RETGIF'])
						return
				except:
					pass

				if gifloc.match(tokens[1]):
					net_engine.sock_write(sock, cfg['CTCGIF_RET_BUFF'])
					net_engine.sock_write(sock, cfg['CTCGIF'])


				# Find out if we need to proxy...

				if (not len(cfg['PARENT_PROXY_HOST'])) or (cfg['NOPROXY']):
					url_tuple = urlparse.urlparse(tokens[1])
					host_port = url_tuple[1]
					if (string.find(host_port, ':') != -1):
						occ = string.count(host_port, ':')
						atocc = string.count(host_port, '@')
						if occ == 1 and atocc == 0:
							rem_srv, rem_port = string.split(host_port, ':')
							try:
								rem_port = string.atoi(rem_port)
							except:
								rem_port = 80
						elif occ == 1 and atocc == 1:
							rem_srv, rem_port = string.split(host_port, '@')[1], 80
						elif occ == 2:
							rem_srv, rem_port = string.split(string.split(host_port, '@')[1], ':')
							try:
								rem_port = string.atoi(rem_port)
							except:
								rem_port = 80
					else:
						rem_srv, rem_port = host_port, 80

				do_proxy = 1
				if cfg['NOPROXY']:
					for skip_host in cfg['NOPROXY']:
						expression = "^.*" + skip_host + "$"
						skip_re = re.compile(expression)
						if skip_re.match(rem_srv):
							do_proxy = None
				if not len(cfg['PARENT_PROXY_HOST']):
					do_proxy = None

				if not do_proxy:
					URL = url_tuple[2]
					if len(url_tuple[3]):
						URL = URL + ';' + url_tuple[3]
					if len(url_tuple[4]):
						URL = URL + '?' + url_tuple[4]
					buff = string.replace(buff, tokens[1], URL)

				else:
					rem_srv = cfg['PARENT_PROXY_HOST']
					rem_port = cfg['PARENT_PROXY_PORT']

				server, msg = net_engine.new_sock_client(rem_srv, rem_port)
				if server == None:
					del(pairs[sock])
					net_engine.sock_write(sock, self.SocketErrorMsg(msg, rem_srv, rem_port))
					net_engine.sock_close(sock)
					return
				else:
					pairs[server] = sockpair.SockPair(sockpair.SERVER)
					pairs[server].pair = sock
					pairs[sock].pair = server
					net_engine.sock_write(server, buff)
			
			else: # More data from POST or PUT operation
				other_end = pairs[sock].pair
				if other_end == None: 
					# Server closed connection before getting all data
					del(pairs[sock])
					net_engine.sock_close(sock)
				else:
					net_engine.sock_write(other_end, buff)

		return


	def onClose(self, sock):
		other_end = self.pairs[sock].pair
		if other_end == None:
			pass
		else:
			del(self.pairs[other_end])
			self.neteng.sock_close(other_end)
			
		del(self.pairs[sock])

		return


	def onLoopEnd(self):

		return


	# Function to return network error messages to the browser
	def SocketErrorMsg(self, msg, rem_srv, rem_port):

		err_msg = self.conf['ERR_MSG']
		err_msg = string.replace(err_msg, '%SERVER', rem_srv)
		err_msg = string.replace(err_msg, '%PORT', `rem_port`)
		try:
			errstr = msg[1]
		except:
			try: 
				errstr = msg[0]
			except:
				errstr = 'Unknown error'
		err_msg = string.replace(err_msg, '%ERROR', errstr)
		err_msg = string.replace(err_msg, '%DATE', time.ctime(time.time()))
		err_msg_size = len(err_msg)
		return_buff = """HTTP/1.1 403 ERROR
Server: Cut The Crap\nContent-Length: """ + `err_msg_size` + """
Content-Type: text/html
Proxy-Connection: Close

"""
		return_buff = return_buff + err_msg

		return return_buff

