# This file is part of 'Cut The Crap (CTC)'
# Copyright 1999 Constantinos A. Kotsokalis <ckotso@cs.ntua.gr>
#
# CTC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# CTC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CTC; see the file COPYING. If not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

import sys
import string
import re
import os
import types
import acl

TYPE_PORT = 1
TYPE_INT = 2
TYPE_STRING = 3
TYPE_FILE = 4
TYPE_ACL = 5
TYPE_NOPROXY = 6

class read_conf:

	def __init__(self, cfg_file, errobj):
		self.errobj = errobj

		if cfg_file != None:
			try:
				cf = open(cfg_file, "r")
			except:
				errobj.eprint("Cannot open configuration file "+cfg_file+"\n", None)
				sys.exit(0)
			self.lines = cf.readlines()
			cf.close()
		else:
			self.lines = []
	
		self.conf = self.cfg_convert(self.parse_cfg())


	# This method parses the configuration file and returns settings as
	# a dictionary
	def parse_cfg(self):
		lines = self.lines
		errobj = self.errobj

		cfg = self.startup_values()

		if len(lines):
			for line in lines:
				line = string.strip(line)
				if (len(line) == 0) or re.search("\s*#", line):
					continue
				if string.find(line, '#'):
					line = string.split(line, '#')[0]
				tokens = re.split('\s+', line)
				if (tokens[0] == 'BIND_PORT'):
					if len(tokens) == 1:
						errobj.eprint("Usage: BIND_PORT <port_num>", None)
					else:
						if self.valid(tokens[1], TYPE_PORT):
							cfg['BIND_PORT'] = string.atoi(tokens[1])
						else:
							errobj.eprint(`tokens[1]` + " is not a valid port number", None)
				elif (tokens[0] == 'NO_LOGGING'):
					if len(tokens) == 1:
						errobj.eprint("Usage: NO_LOGGING <0 or 1>", None)
					else:
						if self.valid(tokens[1], TYPE_INT):
							if tokens[1] == '1':
								cfg['NO_LOGGING'] = 1
							else:
								cfg['NO_LOGGING'] = 0
						else:
							errobj.eprint(`tokens[1]` + " is not a valid option -- must be 0 or 1", None)
				elif (tokens[0] == 'BLOCK_POPUPS'):
					if len(tokens) == 1:
						errobj.eprint("Usage: BLOCK_POPUPS <0 or 1>", None)
					else:
						if self.valid(tokens[1], TYPE_INT):
							if tokens[1] == '1':
								cfg['BLOCK_POPUPS'] = 1
							else:
								cfg['BLOCK_POPUPS'] = 0
						else:	
							errobj.eprint(`tokens[1]` + " is not a valid option -- must be 0 or 1", None)
				elif (tokens[0] == 'DNS_CHECK_SEC'):
					if len(tokens) == 1:
						errobj.eprint("Usage: DNS_CHECK_SEC <number of seconds to refresh DNS cache>", None)
					else:
						if self.valid(tokens[1], TYPE_INT):
							try:
								cfg['DNS_CHECK_SEC'] = string.atoi(tokens[1])
							except:
								cfg['DNS_CHECK_SEC'] = string.atol(tokens[1])
						else:
							errobj.eprint(`tokens[1]` + " is not a valid integer or long number", None)
				elif (tokens[0] == 'ZAPLET_DIR'):
					if len(tokens) == 1:
						errobj.eprint("Usage: ZAPLET_DIR <directory containing zaplets>", None)
					else:
						if self.valid(tokens[1], TYPE_FILE):
							cfg['ZAPLET_DIR'] = tokens[1]
						else:
							errobj.eprint(`tokens[1]` + " is not a valid pathname", None)
				elif (tokens[0] == 'NAME_CACHE'):
					if len(tokens) == 1:
						errobj.eprint("Usage: NAME_CACHE <DNS cache filename>", None)
					else:
						if self.valid(tokens[1], TYPE_FILE):
							cfg['NAME_CACHE'] = tokens[1]
						else:
							errobj.eprint(`tokens[1]` + " is not a valid pathname", None)
				elif (tokens[0] == 'PARENT_PROXY_HOST'):
					if len(tokens) == 1:
						errobj.eprint("Usage: PARENT_PROXY_HOST <proxy server to redirect queries to>", None)
					else:
						if self.valid(tokens[1], TYPE_STRING):
							cfg['PARENT_PROXY_HOST'] = tokens[1]
						else:
							errobj.eprint(`tokens[1]` + " is not a valid hostname", None)
				elif (tokens[0] == 'PARENT_PROXY_PORT'):
					if len(tokens) == 1:
						errobj.eprint("Usage: PARENT_PROXY_PORT <port of proxy server to redirect queries to>", None)
					else:
						if self.valid(tokens[1], TYPE_PORT):
							cfg['PARENT_PROXY_PORT'] = string.atoi(tokens[1])
						else:
							errobj.eprint(`tokens[1]` + " is not a valid port", None)
				elif (tokens[0] == 'RETURN_GIF'):
					if len(tokens) == 1:
						errobj.eprint("Usage: RETURN_GIF <pathname of GIF to return instead of ads>", None)
					else:
						if self.valid(tokens[1], TYPE_FILE):
							cfg['RETURN_GIF'] = tokens[1]
						else:
							errobj.eprint(`tokens[1]` + " is not a valid pathname", None)
				elif (tokens[0] == 'CTC_GIF'):
					if len(tokens) == 1:
						errobj.eprint("Usage: CTC_GIF <pathname of CTC's logo>", None)
					else:
						if self.valid(tokens[1], TYPE_FILE):
							cfg['CTC_GIF'] = tokens[1]
						else:
							errobj.eprint(`tokens[1]` + " is not a valid pathname", None)
				elif (tokens[0] == 'ERROR_HTML_FILE'):
					if len(tokens) == 1:
						errobj.eprint("Usage: ERROR_HTML_FILE <pathname of custom HTML file to return on errors>", None)
					else:
						if self.valid(tokens[1], TYPE_FILE):
							cfg['ERROR_HTML_FILE'] = tokens[1]
						else:
							errobj.eprint(`tokens[1]` + " is not a valid pathname", None)
				elif (tokens[0] == 'ACL'):
					if len(tokens) < 3:
						errobj.eprint("Usage: ACL <ALLOW|DENY> <DOMAIN|ALL>", None)
					else:
						if self.valid(tokens[1], TYPE_ACL):
							if tokens[1] == 'ALLOW':
								if tokens[2] == 'ALL':
									cfg['ACL'].all_allow()
								else:
									cfg['ACL'].add_allow(tokens[2])
							elif tokens[1] == 'DENY':
								if tokens[2] == 'ALL':
									cfg['ACL'].all_deny()
								else:
									cfg['ACL'].add_deny(tokens[2])
						else:
							errobj.eprint(`tokens[1]` + " is not a valid directive", None)

				elif (tokens[0] == 'NOPROXY'):
					if len(tokens) <> 2:
						errobj.eprint("Usage: NOPROXY <host|domain>")
					else:
						if self.valid(tokens[1], TYPE_NOPROXY):
							cfg['NOPROXY'].append(tokens[1])
						else:
							errobj.eprint('tokens[1]' + " is not a valid domainname", None)

				else:
					errobj.eprint("Unknown option: "+tokens[0]+"\n", None)

		return cfg


	def startup_values(self):
		cfg = {}
		cfg['BIND_PORT'] = 5050
		cfg['PARENT_PROXY_HOST'] = ''
		cfg['PARENT_PROXY_PORT'] = 8080
		cfg['RETURN_GIF'] = "/usr/local/lib/ctc/white.gif"
		cfg['CTC_GIF'] = "/usr/local/lib/ctc/CTC.gif"
		cfg['ERROR_HTML_FILE'] = "/usr/local/lib/ctc/ctc_error.html"
		cfg['ZAPLET_DIR'] = "/usr/local/lib/ctc/zaplets"
		cfg['NAME_CACHE'] = None
		cfg['DNS_CHECK_SEC'] = 1800
		cfg['BLOCK_POPUPS'] = 0
		cfg['ACL'] = acl.acl(self.errobj)
		cfg['NO_LOGGING'] = 0
		cfg['NOPROXY'] = []

		cfg['VERSION'] = '1.0.2'
		cfg['VERSION_NAME'] = 'Highlands'
		cfg['BUFF_SIZE'] = 8192

		return cfg


	def cfg_convert(self, cfg):
		errobj = self.errobj

		tmpval = os.stat(cfg['RETURN_GIF'])
		RETURN_GIF_SIZE = tmpval[6]
		locf = open(cfg['RETURN_GIF'], 'r')
		cfg['RETGIF'] = locf.read(RETURN_GIF_SIZE)
		locf.close()
		del cfg['RETURN_GIF']

		tmpval = os.stat(cfg['CTC_GIF'])
		CTC_GIF_SIZE = tmpval[6]
		locf = open(cfg['CTC_GIF'], 'r')
		cfg['CTCGIF'] = locf.read(CTC_GIF_SIZE)
		locf.close()
		del cfg['CTC_GIF']

		tmpval = os.stat(cfg['ERROR_HTML_FILE'])
		ERROR_HTML_SIZE = tmpval[6]
		locf = open(cfg['ERROR_HTML_FILE'], 'r')
		cfg['ERR_MSG'] = locf.read(ERROR_HTML_SIZE)
		locf.close()
		del cfg['ERROR_HTML_FILE']

		cfg['ADV_RET_BUFF'] = """HTTP/1.1 200 OK
Server: Cut The Crap
Content-Length: """ + `RETURN_GIF_SIZE` + """
Proxy-Connection: close
Content-Type: image/gif

"""
		cfg['CTCGIF_RET_BUFF'] = """HTTP/1.1 200 OK
Server: Cut The Crap
Content-Length: """ + `CTC_GIF_SIZE` + """
Proxy-Connection: close
Content-Type: image/gif

"""

		return cfg


	def valid(self, token, datatype):
		if datatype == TYPE_PORT:
			if type(token) != types.StringType:
				return 0
			else:
				try:
					token = string.atoi(token)
				except:
					return 0
				if token < 0 or token > 65535:
					return 0
			return TYPE_PORT
		elif datatype == TYPE_INT:
			if type(token) != types.StringType:
				return 0
			else:
				try:
					token = string.atoi(token)
				except:
					try:
						token = string.atol(token)
					except:
						return 0
			return TYPE_INT
		elif datatype == TYPE_FILE:
			if type(token) != types.StringType:
				return 0
			else:
				try:
					tmp = os.stat(token)
				except OSError:
					return 0
			return TYPE_FILE
		elif datatype == TYPE_STRING:
			if type(token) != types.StringType:
				return 0
			return 1
		elif datatype == TYPE_ACL:
			if type(token) != types.StringType:
				return 0
			else:
				if token != 'DENY' and token != 'ALLOW':
					return 0
				else:
					return 1
		elif datatype == TYPE_NOPROXY:
			if type(token) != types.StringType:
				return 0
			else:
				return 1


